module sfx_esm_param
      !< @brief ESM surface flux model parameters
      !< @details  all in SI units

      ! modules used
      ! --------------------------------------------------------------------------------
      use sfx_phys_const
      ! --------------------------------------------------------------------------------

      ! directives list
      ! --------------------------------------------------------------------------------
      implicit none
      ! --------------------------------------------------------------------------------


      !< von Karman constant [n/d]
      real, parameter :: kappa = 0.40
      !< inverse Prandtl (turbulent) number in neutral conditions [n/d]
      real, parameter :: Pr_t_0_inv = 1.15
      !< inverse Prandtl (turbulent) number in free convection [n/d]
      real, parameter :: Pr_t_inf_inv = 3.5

      !< stability function coeff. (unstable) [= g4 & g10 in deprecated code]
      real, parameter :: alpha_m = 16.0
      real, parameter :: alpha_h = 16.0
      real, parameter :: alpha_h_fix = 1.2

      !< stability function coeff. (stable)
      real, parameter :: beta_m = 4.7
      real, parameter :: beta_h = beta_m

      !< --- max Ri-bulk value in stable case ( < 1 / beta_m )
      real, parameter ::  Rib_max = 0.9 / beta_m

end module sfx_esm_param
